@@ -30,11 +30,8 @@
 
 #include <kaboutdata.h>
 #include <kconfig.h>
-#include <kfiledialog.h>
 #include <kfileitem.h>
 #include <kicondialog.h>
-#include <kdirselectdialog.h>
-#include <kcolordialog.h>
 #include <kcolormimedata.h>
 #include <kwindowsystem.h>
 #include <kiconloader.h>
@@ -43,11 +40,14 @@
 #include <QApplication>
 #include <QDate>
 #include <QClipboard>
+#include <QColorDialog>
 #include <QDBusServiceWatcher>
 #include <QDebug>
 #include <QDesktopWidget>
+#include <QFileDialog>
 #include <QUrl>
 #include <QProcess>
+#include <QRegularExpression>
 #include <QTimer>
 
 #include <iostream>
@@ -212,6 +212,42 @@ static KGuiItem configuredContinue(const QString &text)
   return KGuiItem( text, "arrow-right" );
 }
 
+static void setFileDialogFilter(QFileDialog &dlg, const QString &filter)
+{
+    if (filter.contains("*")) {
+        QString qtFilter = filter;
+        dlg.setNameFilter(qtFilter.replace('|','\n'));
+    } else if (!filter.isEmpty()) {
+        dlg.setMimeTypeFilters(filter.trimmed().split(' '));
+    }
+}
+
+// taken from qfiledialog.cp
+static QString initialSelection(const QUrl &url)
+{
+    if (url.isEmpty()) {
+        return QString();
+    }
+    if (url.isLocalFile()) {
+        QFileInfo info(url.toLocalFile());
+        if (!info.isDir()) {
+            return info.fileName();
+        } else {
+            return QString();
+        }
+    }
+    // With remote URLs we can only assume.
+    return url.fileName();
+}
+
+static QUrl initialDirectory(const QUrl &url)
+{
+    if (url.isLocalFile() && QFileInfo(url.toLocalFile()).isDir()) {
+        return url;
+    }
+    return url.adjusted(QUrl::RemoveFilename);
+}
+
 int main(int argc, char *argv[])
 {
     // Bug 373677: Qt removes various arguments it treats internally (such as title and icon) from args
@@ -229,7 +265,7 @@ int main(int argc, char *argv[])
     app.setAttribute(Qt::AA_UseHighDpiPixmaps, true);
 
     KAboutData aboutData( "kdialog", i18n("KDialog"),
-            "1.0", i18n( "KDialog can be used to show nice dialog boxes from shell scripts" ),
+            "2.0", i18n( "KDialog can be used to show nice dialog boxes from shell scripts" ),
             KAboutLicense::GPL,
             i18n("(C) 2000, Nick Thompson"));
     aboutData.addAuthor(i18n("David Faure"), i18n("Current maintainer"),"faure@kde.org");
@@ -513,7 +549,6 @@ int main(int argc, char *argv[])
                                                        passiveicon,
                                                        (QWidget*)0UL, // parent
                                                        timeout );
-        KDialog::centerOnScreen( popup );
         QTimer *timer = new QTimer();
         QObject::connect( timer, SIGNAL(timeout()), qApp, SLOT(quit()) );
         QObject::connect( popup, SIGNAL(clicked()), qApp, SLOT(quit()) );
@@ -588,6 +623,7 @@ int main(int argc, char *argv[])
             cout << result.toLocal8Bit().data() << endl;
             return retcode ? 0 : 1;
         }
+        cerr << qPrintable(i18n("Syntax: --combobox <text> item [item] ...")) << endl;
         return -1;
     }
 
@@ -609,6 +645,7 @@ int main(int argc, char *argv[])
             }
             return retcode ? 0 : 1;
         }
+        cerr << qPrintable(i18n("Syntax: --menu text [tag item] [tag item] ...")) << endl;
         return -1;
     }
 
@@ -626,15 +663,16 @@ int main(int argc, char *argv[])
             const bool retcode = Widgets::checkList(0, title, text, list, separateOutput, result);
 
             for (int i=0; i<result.count(); i++)
-                if (!result.at(i).toLocal8Bit().isEmpty()) {
+                if (!result.at(i).isEmpty()) {
                     cout << result.at(i).toLocal8Bit().data() << endl;
                 }
             return retcode ? 0 : 1;
         }
+        cerr << qPrintable(i18n("Syntax: --checklist text [tag item on/off] [tag item on/off]")) << endl;
         return -1;
     }
 
-    // --radiolist text width height menuheight [tag item status]
+    // --radiolist text [tag item status] ...
     if (parser.isSet("radiolist")) {
         QStringList list;
         if (args.count() >= 3) {
@@ -648,43 +686,66 @@ int main(int argc, char *argv[])
             cout << result.toLocal8Bit().data() << endl;
             return retcode ? 0 : 1;
         }
+        cerr << qPrintable(i18n("Syntax: --radiolist text [tag item on/off] ...")) << endl;
         return -1;
     }
 
     // getopenfilename [startDir] [filter]
-    if (parser.isSet("getopenfilename")) {
+    // getopenurl [startDir] [filter]
+    if (parser.isSet("getopenfilename") || parser.isSet("getopenurl")) {
         QString startDir = args.count() > 0 ? args.at(0) : QString();
         const QUrl startUrl = QUrl::fromUserInput(startDir);
         QString filter;
         if (args.count() > 1)  {
             filter = Utils::parseString(args.at(1));
         }
-        KFileDialog dlg( startUrl, filter, 0 );
-        dlg.setOperationMode( KFileDialog::Opening );
+        const bool multiple = parser.isSet("multiple");
 
-        if (parser.isSet("multiple")) {
-            dlg.setMode(KFile::Files | KFile::LocalOnly);
+        QFileDialog dlg;
+        dlg.setAcceptMode(QFileDialog::AcceptOpen);
+        dlg.setDirectoryUrl(initialDirectory(startUrl));
+        dlg.selectFile(initialSelection(startUrl));
+
+        if (multiple) {
+            dlg.setFileMode(QFileDialog::ExistingFiles);
         } else {
-            dlg.setMode(KFile::File | KFile::LocalOnly);
+            dlg.setFileMode(QFileDialog::ExistingFile);
         }
+        const bool openUrls = parser.isSet("getopenurl");
+        if (!openUrls) {
+            dlg.setSupportedSchemes({"file"});
+        }
+        setFileDialogFilter(dlg, filter);
         Utils::handleXGeometry(&dlg);
         dlg.setWindowTitle(title.isEmpty() ? i18nc("@title:window", "Open") : title);
-        dlg.exec();
+        if (!dlg.exec()) {
+            return 1; // canceled
+        }
 
-        if (parser.isSet("multiple")) {
-            const QStringList result = dlg.selectedFiles();
-            if ( !result.isEmpty() ) {
-                outputStringList( result, separateOutput );
-                return 0;
+        if (openUrls) {
+            const QList<QUrl> result = dlg.selectedUrls();
+            if (!result.isEmpty()) {
+                if (multiple) {
+                    outputStringList(result, separateOutput);
+                    return 0;
+                } else {
+                    cout << result.at(0).url().toLocal8Bit().data() << endl;
+                    return 0;
+                }
             }
         } else {
-            const QString result = dlg.selectedFile();
+            const QStringList result = dlg.selectedFiles();
             if (!result.isEmpty())  {
-                cout << result.toLocal8Bit().data() << endl;
-                return 0;
+                if (multiple) {
+                    outputStringList(result, separateOutput);
+                    return 0;
+                } else {
+                    cout << result.at(0).toLocal8Bit().data() << endl;
+                    return 0;
+                }
             }
         }
-        return 1; // canceled
+        return 1;
     }
 
 
@@ -698,31 +759,30 @@ int main(int argc, char *argv[])
         if (args.count() > 1)  {
             filter = Utils::parseString(args.at(1));
         }
-        // copied from KFileDialog::getSaveFileName(), so we can add geometry
-        bool specialDir = startDir.startsWith(QLatin1Char(':'));
-        if ( !specialDir ) {
-            KFileItem kfi(startUrl);
-            specialDir = kfi.isDir();
-        }
-        KFileDialog dlg( specialDir ? startUrl : QUrl(), filter, 0 );
-        if ( !specialDir )
-            dlg.setSelection( startDir );
-        dlg.setOperationMode( KFileDialog::Saving );
+        QFileDialog dlg;
+        dlg.setAcceptMode(QFileDialog::AcceptSave);
+        dlg.setFileMode(QFileDialog::AnyFile);
+        dlg.setDirectoryUrl(initialDirectory(startUrl));
+        dlg.selectFile(initialSelection(startUrl));
+        setFileDialogFilter(dlg, filter);
         Utils::handleXGeometry(&dlg);
         dlg.setWindowTitle(title.isEmpty() ? i18nc("@title:window", "Save As") : title);
-        dlg.exec();
+        if (!dlg.exec()) {
+            return 1; // canceled
+        }
 
         if ( parser.isSet("getsaveurl") ) {
-            const QUrl result = dlg.selectedUrl();
-            if ( result.isValid())  {
-                cout << result.url().toLocal8Bit().data() << endl;
+            const QList<QUrl> result = dlg.selectedUrls();
+            if (!result.isEmpty())  {
+                cout << result.at(0).toString().toLocal8Bit().data() << endl;
                 return 0;
             }
         } else { // getsavefilename
-            const QString result = dlg.selectedFile();
+            const QStringList result = dlg.selectedFiles();
             if (!result.isEmpty())  {
-                KRecentDocument::add(QUrl::fromLocalFile(result));
-                cout << result.toLocal8Bit().data() << endl;
+                const QString file = result.at(0);
+                KRecentDocument::add(QUrl::fromLocalFile(file));
+                cout << file.toLocal8Bit().data() << endl;
                 return 0;
             }
         }
@@ -733,66 +793,21 @@ int main(int argc, char *argv[])
     if (parser.isSet("getexistingdirectory")) {
         QString startDir = args.count() > 0 ? args.at(0) : QString();
         const QUrl startUrl = QUrl::fromUserInput(startDir);
-        QString result;
-#ifdef Q_WS_WIN
-        result = QFileDialog::getExistingDirectory( 0, title, startDir,
-                                                    QFileDialog::DontResolveSymlinks |
-                                                    QFileDialog::ShowDirsOnly);
-#else
-        QUrl url;
-        KDirSelectDialog myDialog( startUrl, true, 0 );
 
-
-        Utils::handleXGeometry(&myDialog);
-        if ( !title.isEmpty() )
-            myDialog.setWindowTitle( title );
-
-        if ( myDialog.exec() == QDialog::Accepted )
-            url = myDialog.url();
-
-        if ( url.isValid() )
-            result = url.path();
-#endif
-        if (!result.isEmpty())  {
-            cout << result.toLocal8Bit().data() << endl;
-            return 0;
-        }
-        return 1; // canceled
-    }
-
-    // getopenurl [startDir] [filter]
-    if (parser.isSet("getopenurl")) {
-        QString startDir = args.count() > 0 ? args.at(0) : QString();
-        const QUrl startUrl = QUrl::fromUserInput(startDir);
-
-        QString filter;
-        if (args.count() > 1)  {
-            filter = Utils::parseString(args.at(1));
-        }
-        KFileDialog dlg( startUrl, filter, 0 );
-        dlg.setOperationMode( KFileDialog::Opening );
-
-        if (parser.isSet("multiple")) {
-            dlg.setMode(KFile::Files);
-        } else {
-            dlg.setMode(KFile::File);
-        }
+        QFileDialog dlg;
+        dlg.setFileMode(QFileDialog::DirectoryOnly);
+        dlg.setOption(QFileDialog::ShowDirsOnly, true);
+        dlg.setDirectoryUrl(initialDirectory(startUrl));
+        dlg.selectFile(initialSelection(startUrl));
         Utils::handleXGeometry(&dlg);
-        dlg.setWindowTitle(title.isEmpty() ? i18nc("@title:window", "Open") : title);
-        dlg.exec();
-
-        if (parser.isSet("multiple")) {
-            const QList<QUrl> result = dlg.selectedUrls();
-            if ( !result.isEmpty() ) {
-                outputStringList( result, separateOutput );
-                return 0;
-            }
-        } else {
-            const QUrl result = dlg.selectedUrl();
-            if (!result.isEmpty())  {
-                cout << result.url().toLocal8Bit().data() << endl;
-                return 0;
-            }
+        dlg.setWindowTitle(title.isEmpty() ? i18nc("@title:window", "Select Directory") : title);
+        if (!dlg.exec()) {
+            return 1; // canceled
+        }
+        const QList<QUrl> urls = dlg.selectedUrls();
+        if (!urls.isEmpty()) {
+            cout << qPrintable(urls.at(0).toDisplayString(QUrl::PreferLocalFile)) << endl;
+            return 0;
         }
         return 1; // canceled
     }
@@ -886,25 +901,25 @@ int main(int argc, char *argv[])
 
     // --getcolor
     if (parser.isSet("getcolor")) {
-        KColorDialog dlg((QWidget*)0L, true);
+        QColorDialog dlg;
 
         if (parser.isSet("default")) {
             defaultEntry = parser.value("default");
-            dlg.setColor(defaultEntry);
+            dlg.setCurrentColor(defaultEntry);
         } else {
             QColor color = KColorMimeData::fromMimeData(QApplication::clipboard()->mimeData(QClipboard::Clipboard));
             if (color.isValid()) {
-                dlg.setColor(color);
+                dlg.setCurrentColor(color);
             }
         }
 
         Utils::handleXGeometry(&dlg);
         dlg.setWindowTitle(title.isEmpty() ? i18nc("@title:window", "Choose Color") : title);
 
-        if (dlg.exec() == KColorDialog::Accepted) {
+        if (dlg.exec()) {
             QString result;
-            QRegularExpressionMatch match;
-            if (dlg.color().isValid() && parser.isSet("format")) {
+            const QColor color = dlg.selectedColor();
+            if (color.isValid() && parser.isSet("format")) {
                 bool found = false;
                 QString format = parser.value("format");
                 format.remove(QChar('*')); // stripped out * for safety
@@ -926,9 +941,9 @@ int main(int argc, char *argv[])
                     if (3 == match_count) {
                         found = true;
                         if (match.captured(0).contains(QRegularExpression("[diouxX]"))) {
-                            result = QString::asprintf(format.toUtf8().constData(), dlg.color().red(), dlg.color().green(), dlg.color().blue());
+                            result = QString::asprintf(format.toUtf8().constData(), color.red(), color.green(), color.blue());
                         } else {
-                            result = QString::asprintf(format.toUtf8().constData(), dlg.color().redF(), dlg.color().greenF(), dlg.color().blueF());
+                            result = QString::asprintf(format.toUtf8().constData(), color.redF(), color.greenF(), color.blueF());
                         }
                         break;
                     }
@@ -937,7 +952,7 @@ int main(int argc, char *argv[])
                     cout << "Invalid format pattern";
                 }
             } else {
-                result = dlg.color().name();
+                result = color.name();
             }
             cout << result.toLocal8Bit().data() << endl;
             return 0;
